"""Public exports for alphaxiv-py."""

from ._version import __version__
from .assistant import AssistantAPI
from .client import AlphaXivClient
from .exceptions import AlphaXivError, APIError, AuthRequiredError, ResolutionError
from .types import (
    AssistantContext,
    AssistantMessage,
    AssistantRun,
    AssistantSession,
    AssistantStreamEvent,
    Author,
    Citation,
    CommentAuthor,
    ExploreFilterOptions,
    FeedCard,
    FeedFilterSearchResults,
    Folder,
    FolderPaper,
    HomepageSearchResults,
    ImplementationResource,
    Mention,
    OrganizationResult,
    OverviewStatus,
    OverviewSummary,
    OverviewTranslationStatus,
    Paper,
    PaperComment,
    PaperFullText,
    PaperGroup,
    PaperMetrics,
    PaperOverview,
    PaperResources,
    PaperTextPage,
    PaperTranscript,
    PaperVersion,
    PodcastTranscriptLine,
    ResolvedPaper,
    SearchResult,
    UrlMetadata,
)

__all__ = [
    "APIError",
    "AlphaXivClient",
    "AlphaXivError",
    "AssistantAPI",
    "AssistantContext",
    "AssistantMessage",
    "AssistantRun",
    "AssistantSession",
    "AssistantStreamEvent",
    "AuthRequiredError",
    "Author",
    "Citation",
    "CommentAuthor",
    "ExploreFilterOptions",
    "FeedCard",
    "FeedFilterSearchResults",
    "Folder",
    "FolderPaper",
    "HomepageSearchResults",
    "ImplementationResource",
    "Mention",
    "OrganizationResult",
    "OverviewStatus",
    "OverviewTranslationStatus",
    "OverviewSummary",
    "PaperFullText",
    "Paper",
    "PaperComment",
    "PaperGroup",
    "PaperMetrics",
    "PaperOverview",
    "PaperResources",
    "PaperTranscript",
    "PaperTextPage",
    "PaperVersion",
    "PodcastTranscriptLine",
    "ResolutionError",
    "ResolvedPaper",
    "SearchResult",
    "UrlMetadata",
    "__version__",
]
